<?php 
class ZXDemoMaker extends base_module {
	private $uploads_timelive = 864000;
	private $cache_dir = 'var/cache/';
	private $output_dir = '/out/';

	function __construct() {
		$this->cache_dir = PROJECT_ROOT.$this->cache_dir;
	}

	private function emptyFolder($dir, $timelive = 0) {
		if (substr($dir, -1, 1) != '/') $dir = $dir.'/';
		// Remove old files
		$files  = scandir($dir);
		foreach ($files as $f) {
			if (is_dir($dir.$f)) continue;

			//(fileperms($dir.$f) & 0x0080)
			if ((time() - filemtime($dir.$f) > $timelive) && is_writable($dir.$f)) {
				@unlink($dir.$f);
			}
		}
	}

	function upload($field_name) {
		$this->error_report_type = 'active_form';
		$this->error = false;
		//$this->emptyFolder($this->cache_dir, 86400);
		$this->emptyFolder($this->cache_dir, 3600);
		
		if (!isset($_FILES[$field_name])) {
			$this->error('No file selected: '.$field_name, __FILE__, __LINE__);
			return false;
		}

		$file = $_FILES[$field_name];

		if (!empty($file['error'])) {
			switch($file['error']) {
				case '1':
					$this->error('The uploaded file exceeds the upload_max_filesize directive in php.ini');
					return false;
				case '2':
					$this->error('The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form');
					return false;
				case '3':
					$this->error('The uploaded file was only partially uploaded');
					return false;
				case '4':
					$this->error('No file was uploaded.');
					return false;
				case '6':
					$this->error('Missing a temporary folder');
					return false;
				case '7':
					$this->error('Failed to write file to disk');
					return false;
				case '8':
					$this->error('File upload stopped by extension');
					return false;
				case '999':
				default:
					$this->error('No error code avaiable');
					return false;
			}
		}


		$targetFile =  $this->cache_dir.md5(time().$file['name']);
			
		if (file_exists($targetFile) && !is_writable($targetFile)) {
			$this->error('File exists and can not be overwritten');
			return false;
		}
			
		move_uploaded_file(urldecode($file['tmp_name']), $targetFile);
		chmod($targetFile, 0777);

		return $targetFile;
	}
	
	function generateDemo($params) {
		foreach (array('', 'bin', 'bin/unreal', 'res', 'sources') as $d) {
			if (!file_exists($this->output_dir.$d)) {
				mkdir($this->output_dir.$d);	
			}
		}

		copy(PROJECT_ROOT.'resources/demo_maker/bin/sjasmplus.exe', $this->output_dir.'bin/sjasmplus.exe');
		copy(PROJECT_ROOT.'resources/demo_maker/bin/unreal/unreal.exe', $this->output_dir.'bin/unreal/unreal.exe');
		copy(PROJECT_ROOT.'resources/demo_maker/bin/unreal/slipka.rom', $this->output_dir.'bin/unreal/slipka.rom');
		copy(PROJECT_ROOT.'resources/demo_maker/bin/unreal/unreal.ini', $this->output_dir.'bin/unreal/unreal.ini');
		copy(PROJECT_ROOT.'resources/demo_maker/make.cmd', $this->output_dir.'make.cmd');
		copy(PROJECT_ROOT.'resources/demo_maker/sources/builder.asm', $this->output_dir.'sources/builder.asm');
		
		// Generate source
		$fp = fopen(PROJECT_ROOT.'resources/demo_maker/sources/test.asm.tpl', 'r');
		$source_tpl = fread($fp, filesize(PROJECT_ROOT.'resources/demo_maker/sources/test.asm.tpl'));
		fclose($fp);
		
		$source_tpl = str_replace('%border_color%', $params['border'], $source_tpl);
		$source_tpl = str_replace('%animation_border%', $params['animation_border'], $source_tpl);
		$source_tpl = str_replace('%main_color%', $params['main_color'], $source_tpl);
		
		//-- frames 1
		if ($params['gif1']) {
			$source_tpl = str_replace('%if_anima%', '', $source_tpl);
			
			$main_frames = '';
			foreach ($params['gif1']['frames'] as $f) {
				$main_frames .= "\t\tdb ".$f['duration']." : dw ".$f['function_name']."\n";
			}
			$source_tpl = str_replace('%main_frames%', $main_frames, $source_tpl);

			//-- include parsed GIF
			$includes = '';
			if ($params['gif1']['functions']['content']) {
				$includes = "\t\tinclude \"res/".$params['gif1']['functions']['filename']."\"\n";
				file_put_contents($this->output_dir.'res\\'.$params['gif1']['functions']['filename'],  $params['gif1']['functions']['content']);
			}
			foreach ($params['gif1']['frames'] as $f) {
				$includes .= "\t\tinclude \"res/".$f['filename']."\"\n";
				file_put_contents($this->output_dir.'res\\'.$f['filename'], $f['content']);
			}
			$source_tpl = str_replace('%includes%', $includes, $source_tpl);
		}
		else {
			$source_tpl = str_replace('%if_anima%', ';', $source_tpl);
		}
		
		
		//-- frames 2
		if ($params['gif2']) {
			$source_tpl = str_replace('%if_anima2%', '', $source_tpl);
				
			$main_frames = '';
			foreach ($params['gif2']['frames'] as $f) {
				$main_frames .= "\t\tdb ".$f['duration']." : dw ".$f['function_name']."\n";
			}
			$source_tpl = str_replace('%main_frames2%', $main_frames, $source_tpl);
		
			//-- include parsed GIF
				
			$includes = '';
			if ($params['gif2']['functions']['content']) {
				$includes = "\t\tinclude \"res/".$params['gif2']['functions']['filename']."\"\n";
				file_put_contents($this->output_dir.'res\\'.$params['gif2']['functions']['filename'], $params['gif1']['functions']['content']);
			}
			foreach ($params['gif2']['frames'] as $f) {
				$includes .= "\t\tinclude \"res/".$f['filename']."\"\n";
				file_put_contents($this->output_dir.'res\\'.$f['filename'], $f['content']);
			}
			$source_tpl = str_replace('%includes2%', $includes, $source_tpl);
		}
		else {
			$source_tpl = str_replace('%if_anima2%', ';', $source_tpl);
		}		
		
		//-- frames 3
		if ($params['gif3']) {
			$source_tpl = str_replace('%if_anima3%', '', $source_tpl);
				
			$main_frames = '';
			foreach ($params['gif3']['frames'] as $f) {
				$main_frames .= "\t\tdb ".$f['duration']." : dw ".$f['function_name']."\n";
			}
			$source_tpl = str_replace('%main_frames3%', $main_frames, $source_tpl);
		
			//-- include parsed GIF
				
			$includes = '';
			if ($params['gif3']['functions']['content']) {
				$includes = "\t\tinclude \"res/".$params['gif3']['functions']['filename']."\"\n";
				file_put_contents($this->output_dir.'res\\'.$params['gif3']['functions']['filename'], $params['gif1']['functions']['content']);
			}
			foreach ($params['gif3']['frames'] as $f) {
				$includes .= "\t\tinclude \"res/".$f['filename']."\"\n";
				file_put_contents($this->output_dir.'res\\'.$f['filename'], $f['content']);
			}
			$source_tpl = str_replace('%includes3%', $includes, $source_tpl);
		}
		else {
			$source_tpl = str_replace('%if_anima3%', ';', $source_tpl);
		}

		
		// -- music
		$source_tpl = str_replace('%if_music%', isset($params['music_file']) ? '' : ';', $source_tpl);
		
		
		// -- splash screen
		$source_tpl = str_replace('%if_splash%', isset($params['splash_file']) ? '' : ';', $source_tpl);
		$source_tpl = str_replace('%splash_delay%', isset($params['splash_delay']) ? $params['splash_delay'] : '0', $source_tpl);
		
		// Main bg
		$source_tpl = str_replace('%if_anima_bg%', isset($params['gif_background']) ? '' : ';', $source_tpl);
		
		// -- scroll
		if (isset($params['scroll_text'])) {
			$source_tpl = str_replace('%if_scroll%', '', $source_tpl);
			$source_tpl = str_replace('%scroll_address%', $params['scroll_address'], $source_tpl);
			$source_tpl = str_replace('%scroll_attr%', $params['scroll_attr'], $source_tpl);
			$source_tpl = str_replace('%scroll_color%', $params['scroll_color'], $source_tpl);
			
			copy(PROJECT_ROOT.'resources/demo_maker/sources/scroll.asm', $this->output_dir.'sources/scroll.asm');
			copy(PROJECT_ROOT.'resources/demo_maker/res/'.$params['scroll_font'], $this->output_dir.'res/16x16font');
			file_put_contents($this->output_dir.'res/scroll', iconv("UTF-8", 'cp1251', mb_strtoupper($params['scroll_text'], "UTF-8")));
		}
		else {
			$source_tpl = str_replace('%if_scroll%', ';', $source_tpl);
			$source_tpl = str_replace('%if_scroll_up%', ';', $source_tpl);
			$source_tpl = str_replace('%scroll_color%', '#47', $source_tpl);
		}
		
		// Analyzator main 
		if (isset($params['analyzator_chanel'])) {
			$source_tpl = str_replace('%if_anal%', '', $source_tpl);
			$source_tpl = str_replace('%analyzator_chanel%', $params['analyzator_chanel'], $source_tpl);
			$source_tpl = str_replace('%analyzator_sens%', $params['analyzator_sens'], $source_tpl);
			
			copy(PROJECT_ROOT.'resources/demo_maker/sources/analyzator.asm', $this->output_dir.'sources/analyzator.asm');
		}
		else {
			$source_tpl = str_replace('%if_anal%', ';', $source_tpl);
		}
		
		// Analyzator splash
		if (isset($params['analyzator_splash_chanel'])) {
			$source_tpl = str_replace('%if_anal_splash%', '', $source_tpl);
			$source_tpl = str_replace('%analyzator_splash_chanel%', $params['analyzator_splash_chanel'], $source_tpl);
			$source_tpl = str_replace('%analyzator_splash_sens%', $params['analyzator_splash_sens'], $source_tpl);
			
			copy(PROJECT_ROOT.'resources/demo_maker/sources/analyzator_splash.asm', $this->output_dir.'sources/analyzator_splash.asm');
		}
		else {
			$source_tpl = str_replace('%if_anal_splash%', ';', $source_tpl);
		}
		
		if (isset($params['music_file'])) {
			copy(PROJECT_ROOT.'resources/demo_maker/sources/PTxPlay.asm', $this->output_dir.'sources/PTxPlay.asm');
			copy($params['music_file'], $this->output_dir.'res/music');
		}
		
		if (isset($params['splash_file'])) {
			copy($params['splash_file'], $this->output_dir.'res/splash');
		}

		if (isset($params['gif_background'])) {
			copy($params['gif_background'], $this->output_dir.'res/bg');
		}

		
		file_put_contents($this->output_dir.'sources/test.asm', $source_tpl);
		
		return true;
	}
}